/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import java.util.Date;
import java.util.List;

import org.ow2.bonita.facade.BAMAPI;
import org.ow2.bonita.util.DateUtil;
import org.ow2.bonita.util.EnvTool;

/**
 * 
 * @author Matthieu Chaffotte
 *
 */
public class BAMAPIImpl implements BAMAPI {

	private String queryList;
	
  protected BAMAPIImpl(final String queryList) {
  	this.queryList = queryList;
  }

  private String getQueryList() {
  	return this.queryList;
  }
  
  public List<Integer> getNumberOfExecutingCasesPerDay(Date since) {
    Date now = new Date();
    return EnvTool.getAllQueriers(getQueryList()).getNumberOfExecutingCasesPerDay(since, now);
  }

  public List<Integer> getNumberOfFinishedCasesPerDay(Date since) {
    Date now = new Date();
    return EnvTool.getAllQueriers(getQueryList()).getNumberOfFinishedCasesPerDay(since, now);
  }

  public int getNumberOfOpenSteps() {
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfOpenSteps();
  }

  public List<Integer> getNumberOfOpenStepsPerDay(Date since) {
    Date now = new Date();
    return EnvTool.getAllQueriers(getQueryList()).getNumberOfOpenStepsPerDay(since, now);
  }

  public int getNumberOfOverdueSteps() {
    Date now = new Date();
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfOverdueSteps(now);
  }

  public int getNumberOfStepsAtRisk(int remainingDays) {
    if (remainingDays < 0) {
      throw new IllegalArgumentException("The number of remaining days is negative");
    }
    Date currentDate = new Date();
    Date beginningOfTheDay = DateUtil.getBeginningOfTheDay(currentDate);
    Date atRisk = DateUtil.backTo(beginningOfTheDay, -(remainingDays+1));
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfStepsAtRisk(currentDate, atRisk);
  }

  public int getNumberOfUserOpenSteps() {
    String userId = EnvTool.getUserId();
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfUserOpenSteps(userId);
  }

  public int getNumberOfUserOverdueSteps() {
    Date now = new Date();
    String userId = EnvTool.getUserId();
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfUserOverdueSteps(userId, now);
  }

  public int getNumberOfUserStepsAtRisk(int remainingDays) {
    if (remainingDays < 0) {
      throw new IllegalArgumentException("The number of remaining days is negative");
    }
    String userId = EnvTool.getUserId();
    Date currentDate = new Date();
    Date beginningOfTheDay = DateUtil.getBeginningOfTheDay(currentDate);
    Date atRisk = DateUtil.backTo(beginningOfTheDay, -(remainingDays+1));
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfUserStepsAtRisk(userId, currentDate, atRisk);
  }

  public int getNumberOfFinishedSteps(int priority, Date since) {
    return EnvTool.getAllQueriers(getQueryList()).getNumberOfFinishedSteps(priority, since);
  }

  public int getNumberOfOpenSteps(int priority) {
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfOpenSteps(priority);
  }

  public int getNumberOfUserFinishedSteps(int priority, Date since) {
    String userId = EnvTool.getUserId();
    return EnvTool.getAllQueriers(getQueryList()).getNumberOfUserFinishedSteps(userId, priority, since);
  }

  public int getNumberOfUserOpenSteps(int priority) {
    String userId = EnvTool.getUserId();
    return EnvTool.getJournalQueriers(getQueryList()).getNumberOfUserOpenSteps(userId, priority);
  }
}
