/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.bonita.facade.APIAccessor;
import org.ow2.bonita.facade.BAMAPI;
import org.ow2.bonita.facade.internal.RemoteBAMAPI;
import org.ow2.bonita.identity.auth.DomainOwner;
import org.ow2.bonita.identity.auth.UserOwner;

/**
 * 
 * @author Elias Ricken de Medeiros
 *
 */
public class RemoteBAMAPIImpl implements RemoteBAMAPI{

  private static final long serialVersionUID = 8977547453867812345L;  
  
  protected Map<String, BAMAPI> apis = new HashMap<String, BAMAPI>();

  protected BAMAPI getAPI(final Map<String, String> options) {
  	final String queryList = options.get(APIAccessor.QUERYLIST_OPTION);
  	final String user = options.get(APIAccessor.USER_OPTION);
  	final String domain = options.get(APIAccessor.DOMAIN_OPTION);
  	UserOwner.setUser(user);
  	DomainOwner.setDomain(domain);
  	
    if (!apis.containsKey(queryList)) {
      apis.put(queryList, new StandardAPIAccessorImpl().getBAMAPI(queryList));
    }
    return apis.get(queryList);
  }
  
  public List<Integer> getNumberOfExecutingCasesPerDay(Date since, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfExecutingCasesPerDay(since);
  }

  public List<Integer> getNumberOfFinishedCasesPerDay(Date since, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfFinishedCasesPerDay(since);
  }

  public int getNumberOfOpenSteps(final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfOpenSteps();
  }

  public List<Integer> getNumberOfOpenStepsPerDay(Date since, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfOpenStepsPerDay(since);
  }

  public int getNumberOfOverdueSteps(final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfOverdueSteps();
  }

  public int getNumberOfStepsAtRisk(int remainingDays, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfStepsAtRisk(remainingDays);
  }

  public int getNumberOfUserOpenSteps(final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfUserOpenSteps();
  }

  public int getNumberOfUserOverdueSteps(final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfUserOverdueSteps();
  }

  public int getNumberOfUserStepsAtRisk(int remainingDays, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfUserStepsAtRisk(remainingDays);
  }

  public int getNumberOfFinishedSteps(int priority, Date since, final Map<String, String> options)
      throws RemoteException {
    return getAPI(options).getNumberOfFinishedSteps(priority, since);
  }

  public int getNumberOfOpenSteps(int priority, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfOpenSteps(priority);
  }

  public int getNumberOfUserFinishedSteps(int priority, Date since, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfUserFinishedSteps(priority, since);
  }

  public int getNumberOfUserOpenSteps(int priority, final Map<String, String> options) throws RemoteException {
    return getAPI(options).getNumberOfUserOpenSteps(priority);
  }
}
