/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import java.util.HashMap;
import java.util.Map;

import org.ow2.bonita.facade.APIAccessor;
import org.ow2.bonita.facade.CommandAPI;
import org.ow2.bonita.facade.internal.RemoteCommandAPI;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.identity.auth.DomainOwner;
import org.ow2.bonita.identity.auth.UserOwner;
import org.ow2.bonita.util.Command;

/**
 * 
 * @author Elias Ricken de Medeiros
 *
 */
public class RemoteCommandAPIImpl implements RemoteCommandAPI{

  private static final long serialVersionUID = 8431354001040613714L; 

  protected Map<String, CommandAPI> apis = new HashMap<String, CommandAPI>();

  protected CommandAPI getAPI(final Map<String, String> options) {
  	final String queryList = options.get(APIAccessor.QUERYLIST_OPTION);
  	final String user = options.get(APIAccessor.USER_OPTION);
  	final String domain = options.get(APIAccessor.DOMAIN_OPTION);
  	UserOwner.setUser(user);
  	DomainOwner.setDomain(domain);
  	
    if (!apis.containsKey(queryList)) {
      putAPI(queryList);
    }
    return apis.get(queryList);
  }

	protected void putAPI(final String queryList) {
		apis.put(queryList, new StandardAPIAccessorImpl().getCommandAPI(queryList));
	}

  public <T> T execute(final Command<T> command, final Map<String, String> options) throws Exception {
    return getAPI(options).execute(command);
  }

  public <T> T execute(final Command<T> command, final ProcessDefinitionUUID processUUID, final Map<String, String> options)
    throws Exception {
    return getAPI(options).execute(command, processUUID);
  }
}


