/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.command;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.facade.APIAccessor;
import org.ow2.bonita.facade.IdentityAPI;
import org.ow2.bonita.facade.identity.User;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.util.Command;

/**
 * 
 * @author Nicolas Chabanoles
 *
 */
public class WebAddUserCommand implements Command<User> {

	private static final long serialVersionUID = 3999511024167567152L;
	private String username;
	private String firstName;
	private String lastName;
	private String password;
  private Map<String, List<String>> memberships;
  private Map<String, String> metadata;

	/**
	 * Default constructor.
	 * Update the user definition. The user's password will not be updated unless the {@code updatePassword} is set to true.
	 * If {@code updatePassword} is false then the {@code password} should be null.
	 */
	public WebAddUserCommand(String username,String firstName, String lastName, String password, Map<String,List<String>> memberships, Map<String,String> metadata) {
		this.username = username;
		this.firstName = firstName;
		this.lastName = lastName;
		this.password = password;
		this.memberships = memberships;
		this.metadata = metadata;
	}

	public User execute(Environment environment) throws Exception {
		final APIAccessor accessor = new StandardAPIAccessorImpl();
		final IdentityAPI identityAPI = accessor.getIdentityAPI();
		
		User result;
		result = identityAPI.addUser(username, password, firstName, lastName, null, null, null,metadata);
		Collection<String> ms = new ArrayList<String>();
		String groupUUID;
		for (Entry<String,List<String>> entry : memberships.entrySet()) {
		  groupUUID = entry.getKey();
		  for (String roleUUID : entry.getValue()) {
		    ms.add(identityAPI.getMembershipForRoleAndGroup(roleUUID, groupUUID).getUUID());
      }
      
    }
		identityAPI.setUserMemberships(result.getUUID(), ms);
		
		return identityAPI.getUserByUUID(result.getUUID());
	}

}
